#ifndef _BIT_INDEX_H
#define _BIT_INDEX_H
/**
 * @file
 * @brief Testovací třídy pro indexované bitové operace.
 * 
 * V kódu nepoužito.
 * Lze použít jen jako lokální zápis
 * @code
 * BitIndexable bi (&REGISTER);
 * bi [index] = true;
 * @endcode
 */
/// @class BitIndexable Pole bitů (jen pro nastavení, zápis)
class BitIndexable {
  public:
    /// Konstruktor nastavi bazi
    BitIndexable (volatile uint32_t * p) {
      base    = p;
      index.v = 0;
    }
    /// [] nastavi index
    BitIndexable & operator[] (const uint32_t p) {
      index.v = p;
      return * this;
    }
    /// pouzit po predchozim operatoru []
    BitIndexable & operator=  (const bool p) {
      if (p) base [index.c] |=  (p << index.d);
      else   base [index.c] &= ~(p << index.d);
      return * this;
    }
  private:
    volatile uint32_t * base;    //!< baze
    union {
      struct {
        uint32_t d :  5;         // 2^5 = 32 bit = uint32_t
        uint32_t c : 27;
      };
      uint32_t v;
    }                  index;    //!< index nastaven pomoci []
};
/// @class TwoBitIndexable Pole dvojbitů (jen pro nastavení, zápis)
class TwoBitIndexable {
  public:
    /// Konstruktor nastavi bazi
    TwoBitIndexable (volatile uint32_t * p) {
      base    = p;
      index.v = 0;
    }
    /// [] nastavi index
    TwoBitIndexable & operator[] (const uint32_t p) {
      index.v = p;
      return * this;
    }
    /// pouzit po predchozim operatoru []
    TwoBitIndexable & operator=  (const uint32_t q) {
      register uint32_t p = q & 3U;              // p = 0,1,2,3
//    if (p != 3) base [index.c] &= ~(3U << (2 * index.d));  // takhle je to
//    if (p != 0) base [index.c] |=  (p  << (2 * index.d));  // spravne
      // zjednodusime - zkraceni kodu na puvodni delku, zachovani funkce
      base [index.c] = p << (2 * index.d);
      return * this;
    }
  private:
    volatile uint32_t * base;    //!< baze
    union {
      struct {
        uint32_t d :  4;         // 2^4 = 16, 16 * 2 bit = 32 bit = uint32_t
        uint32_t c : 28;
      };
      uint32_t v;
    }                  index;    //!< index nastaven pomoci []
};

#endif // _BIT_INDEX_H