#include "io.h"
#include "dbgmcu.h"
/// @file
/// @brief Blikání pomocí SysTick.
extern uint32_t _vect_tab; //!< v linker skriptu
/** Základní IO použité v programu.
 *  Nejsou DEFINE, const je čitší, při optimalizaci stejně místo v paměti nezaberou. */
static RCC_TypeDef  * const RCC   = (RCC_TypeDef  *) RCC_BASE;
/// Stejně pro GPIO bránu.
static GPIO_TypeDef * const GPIOX = (GPIO_TypeDef *) LEDPORT_BASE;
/// Místo main() přímo tt. funkce
extern void Reset_Handler (void);
/// Inicializace pinu na portu C jako výstupu
/// @param pin Pin na portu C
static inline void pinGpioCoutInit (const unsigned int pin) {
  RCC  ->AHBENR |= LEDCLK_GPIOEN;       // zavést hodiny
  GPIOX->MODER  |= (1U << 2*pin);       // a nastavit jako výstup
}
/// Manipulace s výstupem - změna log. hodnoty na pinu.
/// @param pin Pin na portu C
static inline void pinGpioCtoggle  (const unsigned int pin) {
  GPIOX->ODR   ^= (1U << pin);          // změnit výstup
}
/// Modrá LED na kitu
const unsigned int LED = LEDPIN;
//  Obsluha přerušení od systémového časovače SysTick
/// Obsluha přerušení od systémového časovače SysTick
void SysTick_Handler (void) {
  pinGpioCtoggle (LED);                 // Jen změní stav LED, blikáme tedy s frekvencí 1 Hz
}
/// main() není potřeba
NAKED void Reset_Handler (void) {
  fillram();
#ifdef ARM_CM4
  EnableDebugOnSleep ();                // Pokud použiji na F4 wfi, pak je dobré povolit debug v tt. módu
  SCB->VTOR = (uint32_t) &_vect_tab;    // Možná zbytečné, ale funkční - umístění vektorů pro F4
#endif // ARM_CM4
  SysTick_Config  (DELAY_SYST);
  pinGpioCoutInit (LED);
  for (;;) {    // Pro zmenšení spotřeby můžeme uP uspat,
    __WFI ();   // probudí se při přerušení od SysTick.
  }
}

