/// @file
/// @brief Vektory včetně SysTick_Handler

/// Typová definice ukazatele na funkci snad trochu zlepší čitelnost.
typedef void (*pHandler)   (void);

/** V linker skriptu, zde již použito.
*/
extern  void _estack       (void);
/// Obsluha vektoru Reset.
extern  void Reset_Handler (void);
/*
 * S tímto zápisem se setkáme u složitějších projektů psaných pro GCC.
 * ALIAS v podstatě znamená, že pokud je vektor např. HardFault_Handler někde dále
 * v programu definován, použije se právě tato definice. Pokud definován není,
 * použije se pro obsluhu právě ta ALIAS funkce, která definována být musí.
 * Jiné překladače to budou mít udělané podobně, ne však úplně stejně. Někdy je
 * (zřejmě ze zvyku, jiný důvod pro to není) tato část psána v jazyce symbolických adres.
 * */
/// Alias pro handler
#define ALIAS(f) __attribute__ ((weak, alias (#f)))
void NMI_Handler                    (void) ALIAS(Default_Handler);
void HardFault_Handler              (void) ALIAS(Default_Handler);
void SVC_Handler                    (void) ALIAS(Default_Handler);
void PendSV_Handler                 (void) ALIAS(Default_Handler);
void SysTick_Handler                (void) ALIAS(Default_Handler);
/// Neúplná tabulka vektorů pevně na začátku paměti. Viz script.ld.
pHandler const Vectors[] = {
  _estack,
  Reset_Handler,                    // Reset handler
  NMI_Handler,                      // The NMI handler
  HardFault_Handler,                // The hard fault handler
  0,                                // Reserved
  0,                                // Reserved
  0,                                // Reserved
  0,                                // Reserved
  0,                                // Reserved
  0,                                // Reserved
  0,                                // Reserved
  SVC_Handler,                      // SVCall handler
  0,                                // Reserved
  0,                                // Reserved
  PendSV_Handler,                   // The PendSV handler
  SysTick_Handler                   // The SysTick handler
  // ... atd.
};
/// Nepoužito
void Default_Handler (void) {
  for (;;);
}
